@@ -1,16 +1,16 @@
 #
 # This file is part of Config-Model-Itself
 #
-# This software is Copyright (c) 2014 by Dominique Dumont.
+# This software is Copyright (c) 2016 by Dominique Dumont.
 #
 # This is free software, licensed under:
 #
 #   The GNU Lesser General Public License, Version 2.1, February 1999
 #
 package Config::Model::Itself ;
-$Config::Model::Itself::VERSION = '1.241';
+$Config::Model::Itself::VERSION = '2.003';
 use Mouse ;
-use Config::Model 2.055;
+use Config::Model 2.076;
 use 5.010;
 
 use IO::File ;
@@ -21,29 +21,135 @@ use File::Find ;
 use File::Path ;
 use File::Basename ;
 use Data::Compare ;
+use Path::Tiny;
+use Mouse::Util::TypeConstraints;
 
 my $logger = Log::Log4perl::get_logger("Backend::Itself");
 
+subtype 'ModelPathTiny' => as 'Object' => where { $_->isa('Path::Tiny') };
+
+coerce 'ModelPathTiny'  => from 'Str'  => via {path($_)} ;
 
 # find all .pl file in model_dir and load them...
 
-has model_object => (is =>'ro', isa =>'Config::Model::Node', required => 1) ;
-has model_dir    => (is =>'ro', isa =>'Str', required => 1 ) ;
-has force_write  => (is =>'ro', isa => 'Bool', default => 0) ;
+around BUILDARGS => sub {
+    my $orig  = shift;
+    my $class = shift;
+    my %args  = @_;
 
-has modifed_classes => (
-    is =>'rw', 
-    isa =>'HashRef[Bool]', 
+    my $legacy = delete $args{model_object};
+    if ($legacy) {
+        $args{config_model} = $legacy->instance->config_model;
+        $args{meta_instance} = $legacy->instance;
+        $args{meta_root} = $legacy;
+    }
+    return $class->$orig( %args );
+};
+
+has 'config_model' => (
+    is => 'ro',
+    isa => 'Config::Model',
+    lazy_build => 1,
+) ;
+
+
+sub _build_config_model {
+    my $self = shift;
+    # don't trigger builders below
+    if ($self->{meta_root}) {
+        return $self->meta_root->instance->config_model;
+    }
+    elsif ($self->{meta_instance}) {
+        return $self->meta_instance->config_model;
+    }
+    else {
+        return Config::Model -> new ( ) ;
+    }
+}
+
+has check        => (is =>'ro', isa => 'Bool', default => 1) ;
+
+has 'meta_instance' => (
+    is =>'ro',
+    isa =>'Config::Model::Instance',
+    lazy_build => 1,
+) ;
+
+sub _build_meta_instance {
+    my $self = shift;
+
+    # don't trigger builders below
+    if ($self->{meta_root}) {
+        return $self->meta_root->instance;
+    }
+    else {
+        # load Config::Model model
+        return $self->config_model->instance (
+            root_class_name => 'Itself::Model' ,
+            instance_name   => 'meta_model' ,
+            check => $self->check,
+        );
+    }
+
+}
+
+has meta_root => (
+    is =>'ro',
+    isa =>'Config::Model::Node',
+    lazy_build => 1,
+) ;
+
+sub _build_meta_root {
+    my $self = shift;
+
+    return $self->meta_instance -> config_root ;
+}
+
+has cm_lib_dir   => (
+    is =>'ro',
+    isa => 'ModelPathTiny',
+    lazy_build => 1,
+    coerce => 1
+) ;
+
+sub _build_cm_lib_dir {
+    my $self = shift;
+    my $p =  path('lib/Config/Model');
+    if (! $p->is_dir) {
+        $p->mkpath(0, 0755) || die "can't create $p:$!";
+    }
+    return $p;
+}
+
+has force_write  => (is =>'ro', isa => 'Bool', default => 0) ;
+has root_model   => (is =>'ro', isa => 'str');
+
+has modified_classes => (
+    is =>'rw',
+    isa =>'HashRef[Bool]',
     traits => ['Hash'],
     default => sub { {} } ,
     handles => {
         clear_classes => 'clear',
         set_class => 'set',
         class_was_changed => 'get' ,
-        classes_to_write => 'keys' ,
+        class_known => 'exists',
     }
 ) ;
 
+has model_dir => (
+    is => 'ro',
+    isa => 'ModelPathTiny',
+    lazy_build => 1,
+);
+
+sub _build_model_dir {
+    my $self = shift;
+    my $md = $self->cm_lib_dir->child('models');
+    $md->mkpath;
+    return $md;
+}
+
 sub BUILD {
     my $self = shift;
 
@@ -51,58 +157,120 @@ sub BUILD {
         my %args = @_ ;
         my $p = $args{path} || '' ;
         return unless $p =~ /^class/ ;
+        return unless $args{index}; # may be empty when class order is changed
         return if $self->class_was_changed($args{index}) ;
         $logger->info("class $args{index} was modified");
-        
+
         $self->add_modified_class($args{index}) ;
     } ;
-    $self->model_object->instance -> on_change_cb($cb) ;
-    
+    $self->meta_instance -> on_change_cb($cb) ;
+
 }
 
+sub add_tracked_class {
+    my $self = shift;
+    $self->set_class(shift,0) ;
+}
 
 sub add_modified_class {
     my $self = shift;
     $self->set_class(shift,1) ;
 }
 
+sub class_needs_write {
+    my $self = shift;
+    my $name =  shift;
+    return ($self->force_write or not $self->class_known($name) or $self->class_was_changed($name)) ;
+}
+
+sub read_app_files {
+    my $self = shift;
+    my $force_load = shift || 0;
+    my $read_from =  shift ;
+    my $application = shift ;
+
+    my $app_dir = $read_from || $self->model_dir->parent;
+    my %apps;
+    $logger->info("reading app files from ".$app_dir);
+    foreach my $dir ( $app_dir->children(qr/\.d$/) ) {
+
+        $logger->info("reading app dir ".$dir);
+        foreach my $file ( $dir->children() ) {
+            next if $file =~ m!/README!;
+            next if $file =~ /(~|\.bak|\.orig)$/;
+            next if $application and $file->basename ne $application;
+
+            # bad categories are filtered by the model
+            my %data = ( category => $dir->basename('.d') );
+            $logger->info("reading app file ".$file);
+
+            foreach ($file->lines({ chomp => 1})) {
+                s/^\s+//;
+                s/\s+$//;
+                s/#.*//;
+                my ( $k, $v ) = split /\s*=\s*/;
+                next unless $v;
+                $data{$k} = $v;
+            }
+
+            my $appli = $file->basename;
+            $apps{$appli} = $data{model} ;
+
+            $self->meta_root->load_data(
+                data => { application => { $appli => \%data } },
+                check => $force_load ? 'no' : 'yes'
+            ) ;
+        }
+    }
+
+    return \%apps;
+}
 
 sub read_all {
     my $self = shift ;
     my %args = @_ ;
 
-    my $model = delete $args{root_model} 
-      || croak __PACKAGE__," read_all: undefined root_model";
     my $force_load = delete $args{force_load} || 0 ;
+    my $read_from ;
+    my $model_dir ;
+    if ($args{read_from}) {
+        $read_from = path (delete $args{read_from});
+        die "Cannot read from unknown dir ".$read_from unless $read_from->is_dir;
+        $model_dir = $read_from->child('models');
+        die "Cannot read from unknown dir ".$model_dir unless $model_dir->is_dir;
+    }
+
+    my $apps = $self-> read_app_files($force_load, $read_from, delete $args{application});
+
+    my $root_model_arg = delete $args{root_model} || '';
+    my $model = $apps->{$root_model_arg} || $root_model_arg ;
     my $legacy = delete $args{legacy} ;
 
     croak "read_all: unexpected parameters ",join(' ', keys %args) if %args ;
 
-    my $dir = $self->model_dir ;
-    unless (-d $dir ) {
-        croak __PACKAGE__," read_all: unknown model dir $dir";
-    }
+    my $dir = $self->model_dir;
+    $dir->mkpath ;
 
     my $root_model_file = $model ;
     $root_model_file =~ s!::!/!g ;
-    
+    my $read_dir = $model_dir || $dir;
+    $logger->info("searching model files in ".$read_dir);
+
     my @files ;
-    my $wanted = sub { 
-        my $n = $File::Find::name ;
-        push @files, $n if (-f $_ and not /~$/ 
-                            and $n !~ /CVS/
-                            and $n !~ m!.(svn|orig|pod)$!
-                            and $n =~ m!$dir/$root_model_file!
+    my $wanted = sub {
+        push @files, $_ if ( $_->is_file and /\.pl$/
+                            and m!$read_dir/$root_model_file\b!
                            ) ;
     } ;
-    find ($wanted, $dir ) ;
+    $read_dir->visit($wanted, { recurse => 1} ) ;
+
+    my $i = $self->meta_instance ;
 
-    my $i = $self->model_object->instance ;
-    
     my %read_models ;
     my %pod_data ;
     my %class_file_map ;
 
+    my @all_models;
     for my $file (@files) {
         $logger->info("loading config file $file");
 
@@ -113,9 +281,10 @@ sub read_all {
         # @models order is important to write configuration class back in the same
         # order as the declaration
         my @models = $tmp_model -> load ( 'Tmp' , $file ) ;
+        push @all_models, @models;
 
         my $rel_file = $file ;
-        $rel_file =~ s/^$dir\/?//;
+        $rel_file =~ s/^$read_dir\/?//;
         die "wrong reg_exp" if $file eq $rel_file ;
         $class_file_map{$rel_file} = \@models ;
 
@@ -125,6 +294,9 @@ sub read_all {
             my $raw_model =  $tmp_model -> get_raw_model( $model_name ) ;
             my $new_model =  $tmp_model -> get_model( $model_name ) ;
 
+            # track read class to identify later classes added by user
+            $self->add_tracked_class($model_name);
+
             # some modifications may be done to cope with older model styles. If a modif
             # was done, mark the class as changed so it will be saved later
             $self->add_modified_class($model_name) unless Compare($raw_model, $new_model) ;
@@ -133,7 +305,7 @@ sub read_all {
                 foreach my $elt_name (keys %{$new_model->{element}}) {
                     my $moved_data = delete $new_model->{$item}{$elt_name}  ;
                     next unless defined $moved_data ;
-                    $new_model->{element}{$elt_name}{$item} = $moved_data ; 
+                    $new_model->{element}{$elt_name}{$item} = $moved_data ;
                 }
                 delete $new_model->{$item} ;
             }
@@ -146,13 +318,13 @@ sub read_all {
                 my $list  = delete $new_model -> {$what.'_list'} ;
                 my $h     = delete $new_model -> {$what} ;
                 $new_model -> {$what} = [] ;
-                map { 
-                    push @{$new_model->{$what}}, $_, $h->{$_} 
+                map {
+                    push @{$new_model->{$what}}, $_, $h->{$_}
                 } @$list ;
             }
 
             # remove hash key with undefined values
-            map { delete $new_model->{$_} unless defined $new_model->{$_} 
+            map { delete $new_model->{$_} unless defined $new_model->{$_}
                                           and $new_model->{$_} ne ''
               } keys %$new_model ;
             $read_models{$model_name} = $new_model ;
@@ -160,17 +332,19 @@ sub read_all {
 
     }
 
+    $self->{root_model} = $model || (sort @all_models)[0];
+
     # Create all classes listed in %read_models to avoid problems with
     # include statement while calling load_data
-    my $model_obj = $self->model_object ;
-    my $class_element = $model_obj->fetch_element('class') ;
+    my $root_obj = $self->meta_root ;
+    my $class_element = $root_obj->fetch_element('class') ;
     map { $class_element->fetch_with_id($_) } sort keys %read_models ;
 
     #require Tk::ObjScanner; Tk::ObjScanner::scan_object(\%read_models) ;
 
     $logger->info("loading all extracted data in Config::Model::Itself");
     # load with a array ref to avoid warnings about missing order
-    $model_obj->load_data(
+    $root_obj->load_data(
         data => {class => [ %read_models ] },
         check => $force_load ? 'no' : 'yes'
     ) ;
@@ -179,9 +353,9 @@ sub read_all {
     for my $file (@files) {
         $logger->info("loading annotations from file $file");
         my $fh = IO::File->new($file) || die "Can't open $file: $!" ;
-        my @lines = $fh->getlines ;  
+        my @lines = $fh->getlines ;
         $fh->close;
-        $model_obj->load_pod_annotation(join('',@lines)) ;
+        $root_obj->load_pod_annotation(join('',@lines)) ;
 
         my @headers ;
         foreach my $l (@lines) {
@@ -204,15 +378,15 @@ sub read_all {
 sub get_perl_data_model{
     my $self = shift ;
     my %args = @_ ;
-    my $model_obj = $self->{model_object};
+    my $root_obj = $self->{meta_root};
     my $class_name = $args{class_name}
       || croak __PACKAGE__," read: undefined class name";
 
-    my $class_element = $model_obj->fetch_element('class') ; 
+    my $class_element = $root_obj->fetch_element('class') ;
 
     # skip if class was deleted during edition
     return unless $class_element->defined($class_name) ;
-    
+
     my $class_elt = $class_element -> fetch_with_id($class_name) ;
 
     my $model = $class_elt->dump_as_data ;
@@ -227,25 +401,51 @@ sub get_perl_data_model{
     return $model ;
 }
 
+sub write_app_files {
+    my $self = shift;
+
+    my $app_dir = $self->cm_lib_dir;
+    my $app_obj = $self->meta_root->fetch_element('application');
+
+    foreach my $app_name ( $app_obj->fetch_all_indexes ) {
+        my $app = $app_obj->fetch_with_id($app_name);
+        my $cat_dir_name = $app->fetch_element_value( name =>'category' ).'.d';
+        $app_dir->child($cat_dir_name)->mkpath();
+        my $app_file = $app_dir->child($cat_dir_name)->child($app->index_value) ;
+
+        my @lines ;
+        foreach my $name ( $app->children ) {
+            next if $name eq 'category'; # saved as directory above
+
+            my $v = $app->fetch_element_value($name); # need to spit out 0 ?
+            next unless defined $v;
+            push @lines, "$name = $v\n";
+
+        }
+        $logger->info("writing file ".$app_file);
+        $app_file->spew(@lines);
+    }
+
+}
 
 sub write_all {
     my $self = shift ;
     my %args = @_ ;
-    my $model_obj = $self->model_object ;
+    my $root_obj = $self->meta_root ;
     my $dir = $self->model_dir ;
 
     croak "write_all: unexpected parameters ",join(' ', keys %args) if %args ;
 
+    $self->write_app_files;
+
     my $map = $self->{map} ;
 
-    unless (-d $dir ) {
-        mkpath($dir,0, 0755) || die "Can't mkpath $dir:$!";
-    }
+    $dir->mkpath;
 
     # get list of all classes loaded by the editor
-    my %loaded_classes 
-      = map { ($_ => 1); } 
-        $model_obj->fetch_element('class')->fetch_all_indexes ;
+    my %loaded_classes
+      = map { ($_ => 1); }
+        $root_obj->fetch_element('class')->fetch_all_indexes ;
 
     # remove classes that are listed in map
     foreach my $file (keys %$map) {
@@ -255,9 +455,9 @@ sub write_all {
     }
 
     # add remaining classes in map
-    my %new_map =  map { 
-        my $f = $_; 
-        $f =~ s!::!/!g; 
+    my %new_map =  map {
+        my $f = $_;
+        $f =~ s!::!/!g;
         ("$f.pl" => [ $_ ]) ;
     } keys %loaded_classes ;
 
@@ -269,22 +469,22 @@ sub write_all {
         my @data ;
         my @notes ;
         my $file_needs_write = 0;
-        
+
         # check if any a class of a file was modified
         foreach my $class_name (@{$map_to_write{$file}}) {
-            $file_needs_write++ if $self->force_write or $self->class_was_changed($class_name) ;
+            $file_needs_write++ if $self->class_needs_write($class_name);
             $logger->info("file $file class $class_name needs write ",$file_needs_write);
         }
-        
-        next unless $file_needs_write ;    
+
+        next unless $file_needs_write ;
 
         foreach my $class_name (@{$map_to_write{$file}}) {
             $logger->info("writing class $class_name");
-            my $model 
+            my $model
               = $self-> get_perl_data_model(class_name => $class_name) ;
             push @data, $model if defined $model and keys %$model;
-            
-            my $node = $self->{model_object}->grab("class:".$class_name) ;
+
+            my $node = $self->{meta_root}->grab("class:".$class_name) ;
             push @notes, $node->dump_annotations_as_pod ;
             # remove class name from above list
             delete $loaded_classes{$class_name} ;
@@ -292,53 +492,53 @@ sub write_all {
 
         next unless @data ; # don't write empty model
 
-        write_model_file ("$dir/$file", $self->{header}{$file}, \@notes, \@data);
+        write_model_file ($dir->child($file), $self->{header}{$file}, \@notes, \@data);
     }
-    
-    $self->model_object->instance->clear_changes ;
+
+    $self->meta_instance->clear_changes ;
 }
 
 sub write_model_snippet {
     my $self = shift ;
     my %args = @_ ;
-    my $snippet_dir = delete $args{snippet_dir} 
+    my $snippet_dir = delete $args{snippet_dir}
       || croak __PACKAGE__," write_model_snippet: undefined snippet_dir";
-    my $model_file = delete $args{model_file} 
+    my $model_file = delete $args{model_file}
       || croak __PACKAGE__," write_model_snippet: undefined model_file";
     croak "write_model_snippet: unexpected parameters ",join(' ', keys %args) if %args ;
 
-    my $model = $self->model_object->dump_as_data ;
+    my $model = $self->meta_root->dump_as_data ;
     # print (Dumper( $model)) ;
 
-    my @raw_data = @{$model->{class}} ;
+    my @raw_data = @{$model->{class} || []} ;
     while (@raw_data) {
         my ( $class , $data ) = splice @raw_data,0,2 ;
         $data ->{name} = $class ;
- 
+
         # does not distinguish between notes from underlying model or snipper notes ...
-        my @notes = $self->model_object->grab("class:$class")->dump_annotations_as_pod ;
+        my @notes = $self->meta_root->grab("class:$class")->dump_annotations_as_pod ;
         my $class_dir = $class.'.d';
         $class_dir =~ s!::!/!g;
         write_model_file ("$snippet_dir/$class_dir/$model_file", [], \@notes, [ $data ]);
     }
 
-    $self->model_object->instance->clear_changes ;
+    $self->meta_instance->clear_changes ;
 }
 
 sub read_model_snippet {
     my $self = shift ;
     my %args = @_ ;
-    my $snippet_dir = delete $args{snippet_dir} 
+    my $snippet_dir = delete $args{snippet_dir}
       || croak __PACKAGE__," write_model_snippet: undefined snippet_dir";
-    my $model_file = delete $args{model_file} 
+    my $model_file = delete $args{model_file}
       || croak __PACKAGE__," read_model_snippet: undefined model_file";
 
     croak "read_model_snippet: unexpected parameters ",join(' ', keys %args) if %args ;
 
     my @files ;
-    my $wanted = sub { 
+    my $wanted = sub {
         my $n = $File::Find::name ;
-        push @files, $n if (-f $_ and not /~$/ 
+        push @files, $n if (-f $_ and not /~$/
                             and $n !~ /CVS/
                             and $n !~ m!.(svn|orig|pod)$!
                             and $n =~ m!\.d/$model_file!
@@ -346,11 +546,11 @@ sub read_model_snippet {
     } ;
     find ($wanted, $snippet_dir ) ;
 
-    my $class_element = $self->model_object->fetch_element('class') ;
+    my $class_element = $self->meta_root->fetch_element('class') ;
 
     foreach my $load_file (@files) {
         $logger->info("trying to read snippet $load_file");
-    
+
         my $snippet = do $load_file ;
 
         unless ($snippet) {
@@ -369,9 +569,9 @@ sub read_model_snippet {
         # load annotations
         $logger->info("loading annotations from snippet file $load_file");
         my $fh = IO::File->new($load_file) || die "Can't open $load_file: $!" ;
-        my @lines = $fh->getlines ;  
+        my @lines = $fh->getlines ;
         $fh->close;
-        $self->model_object->load_pod_annotation(join('',@lines)) ;
+        $self->meta_root->load_pod_annotation(join('',@lines)) ;
     }
 }
 
@@ -420,7 +620,7 @@ sub list_class_element {
     my $pad  =  shift || '' ;
 
     my $res = '';
-    my $meta_class = $self->{model_object}->fetch_element('class') ;
+    my $meta_class = $self->{meta_root}->fetch_element('class') ;
     foreach my $class_name ($meta_class->fetch_all_indexes ) {
         $res .= $self->list_one_class_element($class_name) ;
     }
@@ -433,7 +633,7 @@ sub list_one_class_element {
     my $pad  =  shift || '' ;
 
     my $res = $pad."Class: $class_name\n";
-    my $meta_class = $self->{model_object}->fetch_element('class')
+    my $meta_class = $self->{meta_root}->fetch_element('class')
        -> fetch_with_id($class_name) ;
 
     my @elts = $meta_class->fetch_element('element')->fetch_all_indexes ;
@@ -463,7 +663,7 @@ sub get_dot_diagram {
     my $self = shift ;
     my $dot = "digraph model {\n" ;
 
-    my $meta_class = $self->{model_object}->fetch_element('class') ;
+    my $meta_class = $self->{meta_root}->fetch_element('class') ;
     foreach my $class_name ($meta_class->fetch_all_indexes ) {
         my $d_class = $class_name ;
         $d_class =~ s/::/__/g;
@@ -471,7 +671,7 @@ sub get_dot_diagram {
         my $elt_list = '';
         my $use = '';
 
-        my $class_obj =  $self->{model_object}->grab(qq!class:"$class_name"!);
+        my $class_obj =  $self->{meta_root}->grab(qq!class:"$class_name"!);
         my @elts =  $class_obj ->grab(qq!element!) ->fetch_all_indexes ;
         foreach my $elt_name ( @elts ) {
             my $of = '';
@@ -489,7 +689,7 @@ sub get_dot_diagram {
             $elt_list .= "- $elt_name ($type$of)\\n";
         }
 
-        $dot .= $d_class 
+        $dot .= $d_class
              .  qq! [shape=box label="$class_name\\n$elt_list"];\n!
              .  $use . "\n";
 
@@ -560,41 +760,27 @@ Config::Model::Itself - Model editor for Config::Model
 
 =head1 VERSION
 
-version 1.241
+version 2.003
 
 =head1 SYNOPSIS
 
- my $meta_model = Config::Model -> new ( ) ;
-
- # load Config::Model model
- my $meta_inst = $model->instance (
-    root_class_name => 'Itself::Model' ,
-    instance_name   => 'meta_model' ,
- );
-
- my $meta_root = $meta_inst -> config_root ;
-
  # Itself constructor returns an object to read or write the data
  # structure containing the model to be edited
- my $rw_obj = Config::Model::Itself -> new(
-    model_object => $meta_root,
-    model_dir => '/path/to/model_files' ,
- ) ;
+ my $meta_model = Config::Model::Itself -> new( ) ;
 
  # now load the model to be edited
- $rw_obj -> read_all( ) ;
+ $meta_model -> read_all( ) ;
 
  # For Curses UI prepare a call-back to write model
- my $wr_back = sub { $rw_obj->write_all();
+ my $wr_back = sub { $meta_model->write_all(); }
 
  # create Curses user interface
- my $dialog = Config::Model::CursesUI-> new
-      (
-       store => $wr_back,
-      ) ;
+ my $dialog = Config::Model::CursesUI-> new (
+      store => $wr_back,
+ ) ;
 
  # start Curses dialog to edit the mode
- $dialog->start( $meta_model )  ;
+ $dialog->start( $meta_model->config_root )  ;
 
  # that's it. When user quits curses interface, Curses will call
  # $wr_back sub ref to write the modified model.
@@ -626,18 +812,26 @@ dedicated to read and write a set of model files.
 
 =head1 Constructor
 
-=head2 new ( model_object => ... , model_dir => ... )
+=head2 new ( [ cm_lib_dir => ... ] )
 
-Creates a new read/write handler. This handler is dedicated to the
-C<model_object> passed with the constructor. This parameter must be a
-L<Config::Model::Node> class.
+Creates a new read/write handler. If no model_object is passed, the required
+objects are created. C<cm_lib_dir> specifies where are the model files (defaults to
+C<./lib/Config/Model>.
+
+C<cm_lib_dir> is either a C<Path::Tiny> object or a string.
+
+By default, this constructor will create all necessary
+C<Config::Model*> objects.  If needed, you can pass already created
+object with options C<config_model> (L<Config::Model> object),
+C<meta_instance> (L<Config::Model::Instance> object) or C<meta_root>
+(L<Config::Model::Node> object).
 
 =head2 Methods
 
-=head1 read_all (  root_model => ... , [ force_load => 1 ] )
+=head1 read_all ( [ root_model => ... ], [ force_load => 1 ] )
 
 Load all the model files contained in C<model_dir> and all its
-subdirectories. C<root_model> is used to filter the classes read. 
+subdirectories. C<root_model> is used to filter the classes read.
 
 Use C<force_load> if you are trying to load a model containing errors.
 
@@ -666,14 +860,14 @@ debugging your configuration model.
 
 =head2 get_dot_diagram
 
-Returns a graphviz dot file that represents the strcuture of the
+Returns a graphviz dot file that represents the structure of the
 configuration model:
 
 =over
 
 =item *
 
-C<include> are represented by solid lines
+C<include> relations are represented by solid lines
 
 =item *
 
@@ -684,7 +878,7 @@ dashed lines. The name of the element is attached to the dashed line.
 
 =head1 SEE ALSO
 
-L<Config::Model>, L<Config::Model::Node>,
+L<Config::Model>, L<Config::Model::Node>, L<Path::Tiny>
 
 =head1 AUTHOR
 
@@ -692,7 +886,7 @@ Dominique Dumont
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2014 by Dominique Dumont.
+This software is Copyright (c) 2016 by Dominique Dumont.
 
 This is free software, licensed under:
 
