@@ -1,7 +1,7 @@
 #
 # This file is part of Config-Model-Itself
 #
-# This software is Copyright (c) 2014 by Dominique Dumont.
+# This software is Copyright (c) 2016 by Dominique Dumont.
 #
 # This is free software, licensed under:
 #
@@ -26,7 +26,7 @@
 #    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 
 package Config::Model::Itself::TkEditUI ;
-$Config::Model::Itself::TkEditUI::VERSION = '1.241';
+$Config::Model::Itself::TkEditUI::VERSION = '2.003';
 use strict;
 use warnings ;
 use Carp ;
@@ -47,34 +47,58 @@ sub ClassInit {
 sub Populate { 
     my ($cw, $args) = @_;
 
-    my $model_dir    = delete $args->{-model_dir} ;
-    my $model_name   = delete $args->{-model_name} ;
+    my $cm_lib_dir    = (delete $args->{-cm_lib_dir})."/models" ;
+    my $model_name   = delete $args->{-model_name} || '';
     my $root_dir     = delete $args->{-root_dir} ; # used to test the edited model
 
-    $args->{'-title'} ||= "config-model-edit $model_name" ;
+    $args->{'-title'} ||= "cme meta edit $model_name" ;
 
     $cw->SUPER::Populate($args) ;
 
-    my $items = [[ qw/command test -command/, sub{ $cw->test_model }],
-		] ;
+    my $items = [[ qw/command test -command/, sub{ $cw->test_model }] ] ;
 
     my $model_menu = $cw->{my_menu}->cascade(-label => 'Model',
 					     -menuitems => $items) ;
-    $cw->{model_dir} = $model_dir ;
+    $cw->{cm_lib_dir} = $cm_lib_dir ;
     $cw->{model_name} = $model_name ;
     $cw->{root_dir} = $root_dir ;
+
+    $cw->show_message("Add a name in Class to create your model") unless $model_name;
 }
 
 sub test_model {
     my $cw = shift ;
 
-    $cw->save_if_yes("save model data ?") ;
+    if ( $cw->{root}->instance->needs_save ) {
+        my $answer = $cw->Dialog(
+            -title          => "save model before test",
+            -text           => "Save model ?",
+            -buttons        => [ qw/yes no cancel/, 'show changes' ],
+            -default_button => 'yes',
+        )->Show;
+
+        if ( $answer eq 'yes' ) {
+            $cw->save( sub {$cw->_launch_test;});
+        }
+        elsif ( $answer eq 'no' ) {
+            $cw->_launch_test;
+        }
+        elsif ( $answer =~ /show/ ) {
+            $cw->show_changes( sub { $cw->test_model } );
+        }
+    }
+    else {
+        $cw->_launch_test;
+    }
+}
+sub _launch_test {
+    my $cw = shift ;
 
     my $testw =  $cw -> {test_widget} ;
     $testw->destroy if defined $testw and Tk::Exists($testw);
 
     # need to read test model from where it was written...
-    my $model = Config::Model -> new(model_dir => $cw->{model_dir}) ;
+    my $model = Config::Model -> new(model_dir => $cw->{cm_lib_dir}) ;
 
     # keep a reference on this object, otherwise it will vanish at the end of this block.
     $cw->{test_model} =  $model ;
