@@ -1,13 +1,13 @@
 #
 # This file is part of Config-Model-Itself
 #
-# This software is Copyright (c) 2013 by Dominique Dumont.
+# This software is Copyright (c) 2016 by Dominique Dumont.
 #
 # This is free software, licensed under:
 #
 #   The GNU Lesser General Public License, Version 2.1, February 1999
 #
-#    Copyright (c) 2007-2011 Dominique Dumont.
+#    Copyright (c) 2007-2015 Dominique Dumont.
 #
 #    This file is part of Config-Model-Itself.
 #
@@ -50,6 +50,26 @@
                 }
             },
 
+            'class' => {
+                type       => 'leaf',
+                value_type => 'uniline',
+                summary    => "Override implementation of configuration node",
+                description =>
+                "Perl class name used to override the default implementation of a configuration node. "
+                ."This Perl class must inherit L<Config::Model::Node>. Use with care.",
+                assert => {
+                    "1_load_class" => {
+                        code => 'not defined $_ or eval{Mouse::Util::load_class($_)}; not $@;',
+                        msg  => 'Error while loading $_ class ',
+                    },
+                    "2_class_inherit" => {
+                        code => 'not defined $_ or $_->isa("Config::Model::Node")',
+                        msg  => 'class $_ must inherit Config::Model::Node',
+                    }
+            },
+
+            },
+
             'element' => {
                 type       => 'hash',
                 level      => 'important',
@@ -61,7 +81,7 @@
                 },
             },
 
-            'include' => {
+            [qw/include include_backend/] => {
                 type  => 'list',
                 cargo => {
                     type       => 'leaf',
@@ -107,8 +127,9 @@
         ],
         'description' => [
             element => "Specify the elements names of this configuration class.",
-            include => "Include the specification of another class into this class.",
+            include => "Include the element description of another class into this class.",
             include_after => "insert the included elements after a specific element",
+            include_backend => "Include the read/write specification of another class into this class.",
             class_description => "Explain the purpose of this configuration class. This description will be re-used to generate the documentation of your configuration class. You can use pod markup to format your description. See L<perlpod> for details.",
             read_config => "Specify the Perl class(es) and function(s) used to read configuration data. The specified function will be tried in sequence to get configuration data. ",
             write_config => "Specify the Perl class and function used to write configuration data.",
@@ -177,20 +198,21 @@
                 type         => 'leaf',
                 class        => 'Config::Model::Itself::BackendDetector',
                 value_type   => 'enum',
-                choice       => [qw/cds_file perl_file ini_file custom/],
+                choice       => [qw/cds_file perl_file custom/],
+                replace   => {
+                    perl => 'perl_file',
+                    ini  => 'IniFile',
+                    ini_file  => 'IniFile',
+                    cds  => 'cds_file',
+                },
                 migrate_from => {
                     formula   => '$old',
                     variables => { old => '- syntax' },
-                    replace   => {
-                        perl => 'perl_file',
-                        ini  => 'ini_file',
-                        cds  => 'cds_file',
-                    },
                 },
                 description => 'specifies the backend to store permanently configuration data.',
                 help => {
                     cds_file => "file with config data string. This is Config::Model own serialisation format, designed to be compact and readable. Configuration filename is made with instance name",
-                    ini_file =>
+                    IniFile =>
 "Ini file format. Beware that the structure of your model must match the limitations of the INI file format, i.e only a 2 levels hierarchy. Configuration filename is made with instance name",
                     perl_file =>
 "file with a perl data structure. Configuration filename is made with instance name",
@@ -233,7 +255,7 @@
                     .'See L<Config::Model::Backend::IniFile/"Arbitrary class name">',
                 warp => {
                     follow => '- backend',
-                    rules  => [ ini_file => { level => 'normal', } ],
+                    rules  => [ IniFile => { level => 'normal', } ],
                 }
             },
 
@@ -245,7 +267,7 @@
                      .'INI class [foo] in element Foo, specify { foo => "Foo" } ',
                 warp => {
                     follow => '- backend',
-                    rules  => [ ini_file => { level => 'normal', } ],
+                    rules  => [ IniFile => { level => 'normal', } ],
                 },
                 cargo => { 
                     type => 'leaf',
@@ -253,25 +275,24 @@
                 },
             },
 
-            'split_list_value' => {
+            ['split_list_value','split_check_list_value'] => {
                 type       => 'leaf',
                 value_type => 'uniline',
                 level      => 'hidden',
-                description => 'Regexp to split values stored in list element. Usually "\s+" or "[,\s]"',
+                description => 'Regexp to split the value read from ini file. Usually "\s+" or "[,\s]"',
                 warp => {
                     follow => '- backend',
-                    rules  => [ ini_file => { level => 'normal', } ],
+                    rules  => [ IniFile => { level => 'normal', } ],
                 }
             },
 
-            'join_list_value' => {
+            ['join_list_value', 'join_check_list_value'] => {
                 type       => 'leaf',
                 value_type => 'uniline',
                 level      => 'hidden',
-                description => 'string to join values from list element. Usually " " or ", "',
                 warp => {
                     follow => '- backend',
-                    rules  => [ ini_file => { level => 'normal', } ],
+                    rules  => [ IniFile => { level => 'normal', } ],
                 }
             },
 
@@ -294,7 +315,7 @@
                 description => "force section to be lowercase",
                 warp => {
                     follow => '- backend',
-                    rules  => [ ini_file => { level => 'normal', } ],
+                    rules  => [ IniFile => { level => 'normal', } ],
                 }
             },
             force_lc_key => {
@@ -305,7 +326,7 @@
                 description => "force key names to be lowercase",
                 warp => {
                     follow => '- backend',
-                    rules  => [ ini_file => { level => 'normal', } ],
+                    rules  => [ IniFile => { level => 'normal', } ],
                 }
             },
             force_lc_value => {
@@ -316,7 +337,7 @@
                 description => "force values to be lowercase",
                 warp => {
                     follow => '- backend',
-                    rules  => [ ini_file => { level => 'normal', } ],
+                    rules  => [ IniFile => { level => 'normal', } ],
                 }
             },
 
@@ -340,10 +361,14 @@
                 upstream_default => '#',
                 warp             => {
                     follow => '- backend',
-                    rules  => [ ini_file => { level => 'normal', } ],
+                    rules  => [ IniFile => { level => 'normal', } ],
                 }
             },
         ],
+        description => [
+            join_list_value => 'string to join list values before writing the entry in ini file. Usually " " or ", "',
+            join_check_list_value => 'string to join checked items names before writing the entry in the ini file. Usually " " or ", "',
+        ],
 
     ],
 
@@ -439,7 +464,7 @@
                 value_type => 'reference' ,
                 refer_to => '- - element' ,
                 description => 'specify where to insert accepted element. This will'
-                 . ' change the behavior but will help generating more consistent '
+                 . ' not change the behavior but will help generating more consistent '
                  . ' user interfaces'
              }
 
