@@ -1,7 +1,7 @@
 #
 # This file is part of Config-Model-Itself
 #
-# This software is Copyright (c) 2014 by Dominique Dumont.
+# This software is Copyright (c) 2016 by Dominique Dumont.
 #
 # This is free software, licensed under:
 #
@@ -54,12 +54,15 @@
                 type       => 'leaf',
                 level      => 'hidden',
                 value_type => 'uniline',
-                summary    => "Override Config::Model::Value:",
+                summary    => "Override implementation of element",
                 description =>
-                  "Perl class name of a child of Config::Model::Value",
+                "Perl class name used to override the implementation of the configuration element. "
+                ."This override Perl class must inherit a Config::Model class that matches the element type, "
+                ."i.e. Config::Model::Value, Config::Model::HashId or Config::Model::ListId. "
+                ."Use with care.",
                 'warp'     => {
-                    follow  => { 't'            => '- type' },
-                    'rules' => [ '$t eq "leaf"' => { level => 'normal', } ]
+                    follow  => { 't'              => '- type' },
+                    'rules' => [ '$t and $t !~ /node/' => { level => 'normal', } ]
                 }
             },
 
